function [LinSolExt] = extendedLinearSolution(LinSol,sol,PQ_tot,Dyn_Syst)
%Function to calculate the Linear Solution of the transverse coordinates in
%terms of the original coefficients [q q_dot]. This is done to be able to
%compare the simulated coefficients using the linear system against the
%ideal calculated nonlinear system. For discrepancies in the inital
%starting point the linearly calculated values should move towards the real
%values.
%The coefficients will be calulated as:
%[Dq Dq_dot] = [L(t); n'(t)]^-1*[I y y_dot]'
%Input...
%LinSol... The solution of the Linear System with the calculated Transverse
%Coordinates.
%Sol... The solution of the non-linear System with the desired value of the
%coefficients.
%Dyn_Syst...    Function handles for the dynamic system.
%ite... current iteration, accesses which part of the step should be
%calculated.
%Output...
%LinSolExt...   The extended Linear solution complete with the calculated
%values for the coefficients of the system. 


%Creating the Extended Linear Solution for Storing
LinSolExt = cell(4,1);

%Calculating the coefficients for the each iteration

for j=1:4,
    %Non-linear System
    th = sol(j).y(:,1);
    dth = sol(j).y(:,2);
    n = length(th);
    dim = length(PQ_tot(1,:,j));
    PQ = PQ_tot(:,:,j);

    %Linear System
    I_t = LinSol{j}.y(:,1);
    y_t = LinSol{j}.y(:,2:5);
    dy_t = LinSol{j}.y(:,6:9);
    l = length(LinSol{j}.y(:,1));

    %Get the coordinates, the 1 is the iteration number. Will increase this to
    %work inside a for-loop. 
    [q,dq,ddq,ddth] = getGeneralizedCoordinates(sol,PQ_tot,Dyn_Syst,j);

    q_est = zeros(n,5);
    dq_est = zeros(n,5);

    for i=1:n,

        [L,N] = getLinear2NonMatrices(th,dth,ddth,dq,ddq,PQ,i);

        L_n = [L;N];

        %Scaling down the transverse coordinates (as they have been calculated
        %using more points than the solution for th)

        tr_ve = zeros(1,9);  

        gam = (l/n);

        ct = gam*i;
        rem = mod(ct,1);
        i_sc = ct-rem;

        if i_sc==0,
            tr_ve = [I_t(1) y_t(1,:) dy_t(1,:)]';
        elseif i==n,
            tr_ve = [I_t(l) y_t(l,:) dy_t(l,:)]';
        else,
            tr_ve = (1-rem)*[I_t(i_sc) y_t(i_sc,:) dy_t(i_sc,:)]' + rem*[I_t(i_sc+1) y_t(i_sc+1,:) dy_t(i_sc+1,:)]';
        end

        %Calculate the Estimated Coefficients from the Linear System!
        Q_delta = L_n\[tr_ve; 0];
        q_est(i,:) = Q_delta(1:5)';
        dq_est(i,:) = Q_delta(6:10)';
    end

    LinSolExt{j}.y = [q_est(:,1) dq_est(:,1)]; %Extracting Theta and dtheta
    LinSolExt{j}.q = [q_est(:,2:5)];
    LinSolExt{j}.dq = dq_est(:,2:5);
    LinSolExt{j}.t = sol(j).t;
    LinSolExt{j}.ye = sol(j).ye;
    LinSolExt{j}.te = sol(j).te;
end

end

    








 










