function [I_y_dy,L_n_2] = switchHandling(LinSol,sol,PQ_tot,Dyn_Syst,ite)
%This function handles the switches in the system to succesfully navigate
%from the end on one solution to the initial start of another. The process
%is as explained in [3] "Transverse Linearization for Impulsive Mechanical
%Systems With One Passive Link. It transforms the Linear Solution
%Calculated and transforms it to a valid starting configuration for the
%next phase.
%Input...
%LinSol, the current Linear Solution of which the ending configuration is
%of the most interest.
%sol, the complete nonlinear solution of which the ending and initial
%configurations before and after a switch is of interest.
%PQ_tot, the total parameters for the parametrizations of the DOF
%Dyn_Syst, function calls for the dynamic system.
%ite, current iteration. To be extended to allow for implementation of all
%4 switches.
%Output...
%I_Y_dy, a vector containing the suitable starting configuration of the
%next step for the parameters of the linear system.


%Extracting Parameters

[l, s, m, I_m, I_r, N, g] = RobotParameters();

%Non-linear System
th = sol(ite).y(:,1);
dth = sol(ite).y(:,2);
n = length(th);
dim = length(PQ_tot(1,:,1));
PQ = PQ_tot(:,:,ite);

%Linear System
I_t = LinSol.y(:,1);
y_t = LinSol.y(:,2:5);
dy_t = LinSol.y(:,6:9);
len = length(LinSol.y(:,1));
 
%Further solutions of the NonLinear System
[q,dq,ddq,ddth] = getGeneralizedCoordinates(sol,PQ_tot,Dyn_Syst,ite);

%Step 1: Converting the linear solution to the non-linear coefficients

[L,N] = getLinear2NonMatrices(th,dth,ddth,dq,ddq,PQ,n);

L_n = [L;N];

tr_ve = [I_t(len) y_t(len,:) dy_t(len,:)]'

%Calculate the Estimated Coefficients from the Linear System!
P = L_n\[tr_ve; 0]

%Step 2: Switch the parameters over to the switching plane

%the m vector
if ite == 1 || ite == 3,
    m = [0,0,0,0,0,1,0,0,0,0]
else,
    m = [l(1)*cos(th(n))+l(2)*cos(th(n)+q(n,1))+l(3)*cos(th(n)+q(n,1)+q(n,2))+...
         +l(4)*cos(th(n)+q(n,1)+q(n,2)+q(n,3))+l(5)*cos(th(n)+q(n,1)+q(n,2)+q(n,3)+q(n,4));
         l(2)*cos(th(n)+q(n,1))+l(3)*cos(th(n)+q(n,1)+q(n,2))+...
         +l(4)*cos(th(n)+q(n,1)+q(n,2)+q(n,3))+l(5)*cos(th(n)+q(n,1)+q(n,2)+q(n,3)+q(n,4));
         l(3)*cos(th(n)+q(n,1)+q(n,2))+...
         +l(4)*cos(th(n)+q(n,1)+q(n,2)+q(n,3))+l(5)*cos(th(n)+q(n,1)+q(n,2)+q(n,3)+q(n,4));
         l(4)*cos(th(n)+q(n,1)+q(n,2)+q(n,3))+l(5)*cos(th(n)+q(n,1)+q(n,2)+q(n,3)+q(n,4));
         l(5)*cos(th(n)+q(n,1)+q(n,2)+q(n,3)+q(n,4));0;0;0;0;0]';
end

%Calculating k_1 OBS: I think the dot products make sense
k_1 = -dot(P,m)/dot(N,m);

V = k_1*N' + P;

%Calculate Impact 
[U1, U2] = getImpactModel(Dyn_Syst,sol,PQ,ite);
 


%Switch the coordinates
F = [1 1 1 1 1;
     0 0 0 0 -1;
     0 0 0 -1 0;
     0 0 -1 0 0;
     0 -1 0 0 0];
 
if ite == 2 || ite == 4,
    V(6:10) = U1\U2*V(6:10);
    V(1:5) = F*V(1:5);
    V(6:10) = F*V(6:10);
end

% if ite == 2 || ite ==4,
% F = [1     1     1     1     1     0     0     0     0     0;
%      0     0     0     0    -1     0     0     0     0     0;
%      0     0     0    -1     0     0     0     0     0     0;
%      0     0    -1     0     0     0     0     0     0     0;
%      0    -1     0     0     0     0     0     0     0     0;
%      0     0     0     0     0     1     1     1     1     1;
%      0     0     0     0     0     0     0     0     0    -1;
%      0     0     0     0     0     0     0     0    -1     0;
%      0     0     0     0     0     0     0    -1     0     0;
%      0     0     0     0     0     0    -1     0     0     0];
% else,
%     F = eye(10);
% end

% V = F*V;

%Step 3: Switch the new parameters over to the new solution plane
if ite ==4,
    nite = 1;
else
    nite = ite+1;
end

th2 = sol(nite).y(:,1);
dth2 = sol(nite).y(:,2);
PQ2 = PQ_tot(:,:,nite);

[q2,dq2,ddq2,ddth2] = getGeneralizedCoordinates(sol,PQ_tot,Dyn_Syst,nite);

[L2,N2] = getLinear2NonMatrices(th2,dth2,ddth2,dq2,ddq2,PQ2,1);

L_n_2 = [L2;N2];

%Calculating k_2
k_2 = -dot(V,N2)/dot(N2,N2);

W =  k_2*N2' + V;

%Step 4: Switching the parameters back into the transverse coordinates
 
[I_y_dy] = (L2*W)';

end

